<?php
/**
*   Billink Payment
*
*   Do not copy, modify or distribute this document in any form.
*
*   @author     Matthijs <matthijs@blauwfruit.nl>
*   @copyright  Copyright (c) 2013-2019 blauwfruit (http://blauwfruit.nl)
*   @license    Proprietary Software
*   @category   payment
*
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class BillinkPayment extends PaymentModule
{
    private $_html = '';
    private $billink_user;
    private $billink_url;
    private $billink_userID;
    private $billink_mode;
    private $billink_version;
    private $billink_workflow;
    private $billink_advise;
    private $billink_max;

    public $hooks = array(
        'displayPayment',
        'displayPaymentEU',
        'paymentOptions',
        'displayAdminOrder',
        'displayHeader',
        'displayBackOfficeHeader',
        'displayOrderConfirmation',
        'hookPaymentOptions',
        'hookPaymentReturn',
        'hookPayment',
    );

    protected $_postErrors = array();

    public $details;
    public $owner;
    public $address;
    public $extra_mail_vars;

    public function __construct()
    {
        $this->name = 'billinkpayment';
        $this->tab = 'payments_gateways';
        $this->version = '3.3.1';
        $this->currencies = false;
        $this->controllers = array('payment', 'validation');
        $this->is_eu_compatible = 1;
        $this->author = 'blauwfruit';
        $this->currencies_mode = 'checkbox';
        $this->bootstrap = true;
        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('Billink');
        $this->description = $this->l('Betaal eenvoudig achteraf met Billink');
        
        require_once 'classes/BillinkAPI.php';

        parent::__construct();

        $this->_init();
    }

    /**
     *  Install
     */
    public function install()
    {
        parent::install();
        foreach ($this->hooks as $hook) {
            if (!$this->registerHook($hook)) {
                return false;
            }
        }
        return true;
    }

    public function hookOrderConfirmation($params)
    {
        # code
    }

    /**
     *  Get content for admin
     */
    public function getContent()
    {
        if (Tools::getValue('save')) {
            $advise = implode(",", Tools::getValue('billink_advise'));
            if (!Configuration::updateValue('BILLINK_URL', (string)(Tools::getValue('billink_url')))
                || !Configuration::updateValue('BILLINK_USR', (string)(Tools::getValue('billink_user')))
                || !Configuration::updateValue('BILLINK_MODE', (int)(Tools::getValue('billink_mode')))
                || !Configuration::updateValue('BILLINK_USER_ID', (string)(Tools::getValue('billink_userID')))
                || !Configuration::updateValue('BILLINK_VERSION', (string)(Tools::getValue('billink_version')))
                || !Configuration::updateValue('BILLINK_WORKFLOW', (string)(Tools::getValue('billink_workflow')))
                || !Configuration::updateValue('BILLINK_ADVISE', (string)($advise))
                || !Configuration::updateValue('BILLINK_STATUS', (int)(Tools::getValue('billink_status')))
                || !Configuration::updateValue('BILLINK_MAX', (int)(Tools::getValue('billink_max')))
            ) {
                echo $this->l('Fout in het opslaan van de instellingen.');
            } else {
                Tools::redirectAdmin('index.php?tab=AdminModules&configure='.$this->name.'&token='.Tools::getValue('token').'&confirm=1');
            }
        }
        $this->_html = $this->_displayForm();
        return $this->_html;
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $cart = $this->context->cart;
        $smarty = $this->context->smarty;
        $customer = new Customer($cart->id_customer);
        $invoice_addres = new Address($cart->id_address_invoice);
        
        /* Check if phone number and birthdate already exists in address */
        $time = strtotime($customer->birthday);
        $birthday_dag = date('d', $time);
        $birthday_maand = (int)date('m', $time);
        $birthday_jaar = date('Y', $time);

        $this->smarty->assign(array(
            'this_path' => $this->_path,
            'telefoon_account' => $invoice_addres->phone,
            'birthday_dag' => $birthday_dag,
            'birthday_maand' => $birthday_maand,
            'birthday_jaar' => $birthday_jaar,
            'current_year' => date('Y'),
            'maanden' => array(1 => 'januari', 2 => 'februari', 3 => 'maart', 4 => 'april', 5 => 'mei', 6 => 'juni', 7 => 'juli', 8 => 'augustus', 9 => 'september', 10 => 'oktober', 11 => 'november', 12 => 'december'),
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'
        ));

        if ($customer->birthday != '' && $customer->birthday != '0000-00-00' && $invoice_addres->phone != '') {
            $geboortedatum = date('d-m-Y', strtotime($customer->birthday));
            $uitkomst = $this->_check('check', $geboortedatum, $invoice_addres->phone);
            if ($uitkomst) {
                $this->smarty->assign('token', $uitkomst);
                $this->smarty->assign('action', $this->context->link->getModuleLink($this->name, 'payment', array(), true));
                $this->smarty->assign('gb', $geboortedatum);
                $this->smarty->assign('tn', $invoice_addres->phone);
                $url = Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/';
                $paymentOptions = array();
                $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
                $newOption
                    ->setCallToActionText('Billink – Achteraf Betalen')
                    ->setForm($this->fetch('module:billinkpayment/views/templates/front/payment_set.tpl'));
                $paymentOptions[] = $newOption;
                return $paymentOptions;
            } else {
                $paymentOptions = array();
                $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
                $newOption
                    ->setCallToActionText($this->l('Op basis van een globale controle kunt u niet achteraf betalen.'));
                $paymentOptions[] = $newOption;
                return $paymentOptions;
            }
        } elseif (Tools::getValue('dag') && Tools::getValue('maand') && Tools::getValue('jaar') && Tools::getValue('telefoon')) {
            $geboortedatum = Tools::getValue('dag') . '-' . Tools::getValue('maand') . '-' . Tools::getValue('jaar');
            $geboortedatum = date('d-m-Y', strtotime($geboortedatum));
            $telefoon = Tools::getValue('telefoon');

            $uitkomst = $this->_check('check', $geboortedatum, $telefoon);

            if ($uitkomst) {
                $this->smarty->assign('token', $uitkomst);
                $this->smarty->assign('gb', $geboortedatum);
                $this->smarty->assign('tn', $telefoon);
                Tools::redirectLink($this->context->link->getModuleLink($this->name, 'payment', array(), true).'?gb='.$geboortedatum.'&tn='.$telefoon.'&scbk='.$uitkomst);
            } else {
                $paymentOptions = array();
                $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
                $newOption
                    ->setCallToActionText($this->l('Op basis van een globale controle kunt u niet achteraf betalen.'));
                $paymentOptions[] = $newOption;
                return $paymentOptions;
            }
        } else {
            $this->smarty->assign(array(
                'this_path' => $this->_path,
                'telefoon_account' => $invoice_addres->phone,
                'birthday_dag' => $birthday_dag,
                'birthday_maand' => $birthday_maand,
                'birthday_jaar' => $birthday_jaar,
                'current_year' => date('Y'),
                'maanden' =>  array(1 => 'januari', 2 => 'februari', 3 => 'maart', 4 => 'april', 5 => 'mei', 6 => 'juni', 7 => 'juli', 8 => 'augustus', 9 => 'september', 10 => 'oktober', 11 => 'november', 12 => 'december'),
                'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
                'action' => '',
                'check' => '1'
            ));

            $paymentOptions = array();
            $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
            $newOption
                ->setCallToActionText('Billink – Achteraf Betalen')
                ->setForm($this->fetch('module:billinkpayment/views/templates/front/payment_set.tpl'));
            $paymentOptions[] = $newOption;
            return $paymentOptions;
        }
    }

    /**
     *  @see hookDisplayOrderConfirmation()
     */
    public function hookPaymentReturn($params)
    {
        # code
    }

    /**
     *  Display confirmation of the Billink payment
     */
    public function hookDisplayOrderConfirmation($params)
    {
        $this->context->smarty->assign(array('reference' => $params['objOrder']->reference));
        return $this->context->smarty->fetch(_PS_MODULE_DIR_.$this->name.'/views/templates/hook/confirmation.tpl');
    }

    /**
     *  Initialize Billink configuration
     */
    private function _init()
    {
        $this->billink_mode = Configuration::get('BILLINK_MODE');
        $this->billink_user = Configuration::get('BILLINK_USR');
        $this->billink_url = Configuration::get('BILLINK_URL');
        $this->billink_userID = Configuration::get('BILLINK_USER_ID');
        $this->billink_version = Configuration::get('BILLINK_VERSION');
        $this->billink_workflow = Configuration::get('BILLINK_WORKFLOW');
        $this->billink_state = Configuration::get('BILLINK_STATUS');
        $this->billink_tax = Configuration::get('BILLINK_TAX');
        $this->billink_max = Configuration::get('BILLINK_MAX');
        $this->billink_advise = explode(",", Configuration::get('BILLINK_ADVISE'));
    }

    /**
     *  Show settings form
     */
    private function _displayForm()
    {
        if (in_array(2, $this->billink_advise)) {
            $advise_2 = 'checked="checked"';
        }

        if (in_array(1, $this->billink_advise)) {
            $advise_1 = 'checked="checked"';
        }

        if (in_array(0, $this->billink_advise)) {
            $advise_0 = 'checked="checked"';
        }

        if (Tools::getValue('confirm')) {
            $this->_html .= '<div class="conf confirm"><img src="/modules/billinkpayment/views/img/ok.gif" alt="' .$this->l('ok').'" /> '.$this->l('Instellingen opgeslagen').'</div>';
        }

        $orderstate = new OrderState();
        $orderstates = $orderstate->getOrderStates($this->context->cookie->id_lang);

        $taxes = Tax::getTaxes();

        $this->_html .= '<h2>'.$this->l('Billink - Koop nu en betaal later').'</h2>';
        $this->_html .= '<p style="line-height: 21px">'.$this->l('Met Billink kunt u op een veilige manier klanten achteraf laten betalen na ontvangst van het product. Een feit is dat consumenten twee keer zo snel overgaan tot een internetaankoop wanneer de optie achteraf betalen wordt geboden. Let op! Het veld geboortedatum is verplicht voor een juiste controle. Verwijder birthday / geboortedatum niet u Prestashop.').'.</p>';
        $this->_html .= '<p>&nbsp;</p><form action="'.$_SERVER['REQUEST_URI'].'" method="post" style="clear: both;"><input type="hidden" value="'.Tools::getValue('token').'" name="token" />';
        $this->_html .= '

                    <fieldset>
                    <legend><img src="'.$this->_path.'logo.gif" />'.$this->l('Instellingen').'</legend>
                    <label for="billink_user">'.$this->l('Gebruikersnaam Billink').':</label>
                    <div class="margin-form">
                            <input type="text" size="50" name="billink_user" id="billink_user" value="'.htmlentities($this->billink_user, ENT_COMPAT, 'UTF-8').'" />
                            <p class="clear">'.$this->l('Billink gebruikersnaam').'</p>
                    </div>

                    <label for="billink_userID">'.$this->l('GebruikersID Billink').':</label>
                    <div class="margin-form">
                            <input type="text" size="50" id="billink_userID" name="billink_userID" value="'.htmlentities($this->billink_userID, ENT_COMPAT, 'UTF-8').'" />
                            <p class="clear">'.$this->l('Billink gebruikers id').'</p>
                    </div>

                    <label for="billink_version">'.$this->l('Billink api versie').':</label>
                    <div class="margin-form">
                            <input type="text" id="billink_version" size="50" name="billink_version" value="'.htmlentities($this->billink_version, ENT_COMPAT, 'UTF-8').'" />
                            <p class="clear">'.$this->l('Billink api versie').'</p>
                    </div>

                    <label for="billink_version">'.$this->l('Workflow nummer').':</label>
                    <div class="margin-form">
                            <input type="text" id="billink_workflow" size="50" name="billink_workflow" value="'.htmlentities($this->billink_workflow, ENT_COMPAT, 'UTF-8').'" />
                            <p class="clear">'.$this->l('Workflow nummer').'</p>
                    </div>

                    <label for="billink_max">'.$this->l('Maximaal orderbedrag').':</label>
                    <div class="margin-form">
                            <input type="text" size="5" id="billink_max" name="billink_max" value="'.htmlentities($this->billink_max, ENT_COMPAT, 'UTF-8').'" />
                            <p class="clear">'.$this->l('Maximaal orderbedrag in hele getallen').'</p>
                    </div>

                    <label for="billink_advise">'.$this->l('Billink mode').':</label>
                    <div class="margin-form">
                            <input type="radio" '.($this->billink_mode ? 'checked="checked"' : '').' name="billink_mode" value="1" id="mode_production">&nbsp;<label class="t" for="mode_production">'.$this->l('Productie').'</label>&nbsp;&nbsp;
                            <input type="radio" '.(!$this->billink_mode ? 'checked="checked"' : '').' name="billink_mode" value="0" id="mode_sandbox">&nbsp;<label class="t" for="mode_sandbox">'.$this->l('Sandbox mode').'</label>&nbsp;&nbsp;
                            <p class="clear">'.$this->l('Kies api mode').'</p>
                    </div>

                    <label for="billink_status">'.$this->l('Standaard Billink order status').':</label>
                    <div class="margin-form">
                            <select name="billink_status" id="billink_status">
                    ';

        foreach ($orderstates as $state) {
            $this->_html .= '<option '.($state['id_order_state'] == $this->billink_state ? 'selected="selected"' : '').' value="'.$state['id_order_state'].'">'.$state['name'].'</option>';
        }

        $this->_html .= '
                            </select>
                            <p class="clear">'.$this->l('Kies een orderstatus voor elke nieuwe bestelling').'</p>
                    </div>

                    <label></label>
                    <input type="submit" name="save" value="'.$this->l('Instellingen opslaan').'" class="button" />

                    </fieldset>
            ';

        return $this->_html;
    }

    /**
     *  displayPayment
     */
    public function hookDisplayPayment($params)
    {
        $plink = $this->context->link->getModuleLink($this->name, 'payment');
        $this->context->smarty->assign(array(
            'plink' => $plink
        ));
        return $this->context->smarty->fetch(_PS_MODULE_DIR_.$this->name.'/views/templates/hook/payment.tpl');
    }
}
