<?php
/**
*   Billink Payment
*
*   Do not copy, modify or distribute this document in any form.
*
*   @author     Matthijs <matthijs@blauwfruit.nl>
*   @copyright  Copyright (c) 2013-2019 blauwfruit (http://blauwfruit.nl)
*   @license    Proprietary Software
*   @category   payment
*
*/

class BillinkPaymentPaymentModuleFrontController extends ModuleFrontController
{
    public $minimalAge = 18;
    public $maximumAge = 120;

    public function setMedia()
    {
        parent::setMedia();
    }

    /**
     * @see FrontController::postProcess()
     */
    public function postProcess()
    {

        parent::postProcess();

        $this->context->controller->addJS(_PS_MODULE_DIR_.$this->module->name.'/views/js/cleave.min.js');
        $this->display_column_left = false;
        $this->display_footer = false;

        $cart = new Cart($this->context->cookie->id_cart);

        if (!$cart->id_customer || !$cart->id_address_delivery || !$cart->id_address_invoice || !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        if (!array_search('billinkpayment', array_column(Module::getPaymentModules(), 'name'))) {
            die($this->module->l('This payment method is not available.', 'validation'));
        }

        $customer = new Customer($this->context->cart->id_customer);
        $address_invoice = new Address($this->context->cart->id_address_invoice);
        $address_delivery = new Address($this->context->cart->id_address_delivery);
        $country = new Country($address_invoice->id_country);
        $cart_total = (float)$cart->getOrderTotal(true, Cart::BOTH);

        if (Tools::isSubmit('isSubmitted')) {

            $unix_birthday = strtotime((string)Tools::getValue('birthday'));
            $age = (time()-$unix_birthday)/(60*60*24*365);
            $customer->birthday = date('Y-m-d', $unix_birthday);
            $address_invoice->phone = Tools::getValue('phone');
            $address_invoice->dni = Tools::getValue('cocnumber');

            $address_invoice->phone = preg_replace('/[^0-9]/', '', $address_invoice->phone);
            if (!$address_invoice->phone) {
                $this->errors[] = $this->module->l('Telefoonnummer is verplicht');
            } elseif (!is_numeric($address_invoice->phone)) {
                $this->errors[] = $this->module->l('Telefoonnummer mag alleen nummers bevatten');
            }

            if ($address_invoice->company) {
                if (!$address_invoice->dni) {
                    $this->errors[] = $this->module->l('KVK-nummer is verplicht', 'payment');
                } elseif (!is_numeric($address_invoice->dni)) {
                    $this->errors[] = $this->module->l('KVK-nummer is niet geen nummer', 'payment');
                }
            } else {

                if (!Tools::getValue('birthday')) {
                    $this->errors[] = $this->module->l('Je hebt geen geboortedatum opgegeven', 'payment');
                } else {
                    if ($age < 0) {
                        $this->errors[] = $this->module->l(sprintf('Is je leeftijd %d? Bent je van de toekomst?', $age), 'payment');
                    } elseif ($age >= $this->maximumAge) {
                        $this->errors[] = $this->module->l(sprintf('Je kunt niet ouder zijn dan %d, dat bestaat niet', $this->maximumAge), 'payment');
                    } elseif ($age <= $this->minimalAge) {
                        $this->errors[] = $this->module->l(sprintf('Je moet minimaal %d jaar oud te zijn', $this->minimalAge), 'payment');
                    }
                }
            }

            try {
                $address_invoice->save();
            } catch (Exception $e) {
                Logger::addLog(sprintf('[%s] %s', $this->module->name, $e->getMessage()));
                $this->errors[] = $this->module->l('Saving address details did not succeed', 'payment');
            }

            try {
                $customer->save();
            } catch (Exception $e) {
                Logger::addLog(sprintf('[%s] %s', $this->module->name, $e->getMessage()));
                $this->errors[] = $this->module->l('Saving customer details did not succeed', 'payment');
            }

            if (!$this->errors) {

                $check = $this->check($customer, $address_invoice, $cart_total);

                if (
                    $check['code'] == 500
                    && isset($check['real_response']->UUID)
                    && (bool)(string)$check['real_response']->UUID
                ) {
                    $UUID = (string)$check['real_response']->UUID;
                } elseif ($check['code'] == 501) {
                    $this->errors[] = $this->module->l('Je komt niet in aanmerking voor achteraf betalen met Billink.');
                } else {
                    $this->errors[] = BillinkAPI::STATES_CHECK[$check['code']];
                }
                
                if (!$this->errors) {
                    $response = $this->order($customer, $address_invoice, $address_delivery, $cart_total, $UUID);
                    if ($response['code'] == 200) {
                        /**
                         *  Payment is successful
                         */
                        $cart = new Cart((int)$this->context->cart->id);
                        $this->module->validateOrder(
                            $cart->id,
                            Configuration::get('PS_OS_PAYMENT'),
                            $cart_total,
                            $this->module->displayName,
                            null,
                            array(),
                            (int)Context::getContext()->currency->id,
                            false,
                            Context::getContext()->customer->secure_key
                        );

                        $cart_id = $cart->id;
                        $module_id = $this->module->id;
                        $order_id = Order::getOrderByCartId((int)$cart->id);
                        $secure_key = Context::getContext()->customer->secure_key;
                        Tools::redirect('index.php?controller=order-confirmation&id_cart='.
                            $cart_id.'&id_module='.$module_id.'&id_order='.$order_id.'&key='.$secure_key);
                    } elseif ($response['code'] == 404) {
                        /**
                         *  Order exists
                         */
                        $this->changeCartId();
                        $this->errors[] = $response['message'];
                    } else {
                        /**
                         *  Error
                         */
                        $this->errors[] = $response['message'];
                    }
                }
            }
        }

        if ($this->errors) {
            Logger::addLog(sprintf('[%s][customer #%d] %s %s', $this->module->name, $customer->id, $customer->email, Tools::jsonEncode($this->errors)));
        }


        $max = time()-60*60*24*365*$this->minimalAge;
        $min = time()-60*60*24*365*$this->maximumAge;
         $this->context->smarty->assign(array(
            'address' => $address_invoice,
            'country' => $country,
            'customer' => $customer,
            'errors' => $this->errors,
            'phone' => $address_invoice->phone,
            'birthday' => array(
                'value' => !($customer->birthday == '0000-00-00') ? date('d-m-Y', strtotime($customer->birthday)) : '',
                'max' => date('Y-m-d', $max),
                'min' => date('Y-m-d', $min),
            ),
            'nbProducts' => $cart->nbProducts(),
            'total' => Tools::displayPrice($cart_total),
            'use_taxes' => 1,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/billinkpayment/'

        ));

        $this->setTemplate('payment.tpl');
    }

    /**
     *  Change order id of the cart
     */
    public function changeCartId()
    {
        $dup = $this->context->cart->duplicate();
        $this->context->cart->delete();
        $this->context->cookie->id_cart = $dup['cart']->id;
    }

    /**
     *  Check - check a customers credability
     * 
     *  @param  (object)    Customer    $customer
     *  @param  (object)    Address     $address_invoice
     *  @param  (float)                 $cart_total 
     */
    public function check(Customer $customer, Address $address_invoice, $cart_total)
    {
        $data = array();
        if ($address_invoice->company) {
            $data['TYPE'] = 'B';
            $data['COMPANYNAME'] = $address_invoice->company;
            $data['CHAMBEROFCOMMERCE'] = $address_invoice->dni;
        } else {
            $data['TYPE'] = 'P';
        }
        $data['ACTION'] = 'Check';
        $data['FIRSTNAME'] = $address_invoice->firstname;
        $data['LASTNAME'] = $address_invoice->lastname;
        $data['INITIALS'] = ucfirst(($address_invoice->firstname[0]));
        $data['SEX'] = $customer->id_gender == 1 ? 'M' : 'V';
        $data['HOUSENUMBER'] = $address_invoice->address2;
        if (preg_match('/[a-zA-Z]$/', trim($address_invoice->address2), $appendix)) {
            $data['HOUSENUMBER'] = str_replace($appendix[0], '', trim($address_invoice->address2));
            $data['HOUSEEXTENSION'] = $appendix[0];
        }

        $data['POSTALCODE'] = $address_invoice->postcode;
        $data['STREET'] = $address_invoice->address1;
        $country = new Country((int)$address_invoice->id_country);
        $data['COUNTRYCODE'] = $country->iso_code;
        $data['CITY'] = $address_invoice->city;
        $data['PHONENUMBER'] = $address_invoice->phone!==''
        ? $address_invoice->phone : $address_invoice->phone_mobile;
        $data['BIRTHDATE'] = date('d-m-Y', strtotime($customer->birthday));
        $data['EMAIL'] = $customer->email;
        $data['ORDERAMOUNT'] = $cart_total;
        $data['IP'] = $_SERVER['REMOTE_ADDR'];
        // $data['HIGHRISK'] = '0';

        $billink = new BillinkAPI;
        return $billink->request('/check', $data);
    }

    /**
     *  Order - Place an order
     *  @param  (object) Customer   $customer 
     *  @param  (object) Address    $address_invoice
     *  @param  (object) Address    $address_delivery
     *  @param  (float)             $cart_total 
     *  @param  (string)            $UUID an identifier key for the credit rating of the customer
     */
    public function order(Customer $customer, Address $address_invoice, Address $address_delivery, $cart_total, $UUID)
    {
        $billink = new BillinkAPI;
        $country = new Country((int)$address_invoice->id_country);

        $data = array();
        if ($address_invoice->company) {
            $data['TYPE'] = 'B';
        } else {
            $data['TYPE'] = 'P';
        }
        $order_number = $this->context->cart->id . (_PS_MODE_DEV_ ? '_' . time() : '');
        $data['ACTION'] = 'Order';
        $data['WORKFLOWNUMBER'] = $billink->workflow;
        $data['CHECKUUID'] = $UUID;
        $data['ORDERNUMBER'] = $order_number;
        $data['DATE'] = date('d-m-Y', time());
        $data['FIRSTNAME'] = $customer->firstname;
        $data['LASTNAME'] = $customer->lastname;
        $data['INITIALS'] = ucfirst(($address_invoice->firstname[0]));
        if (!$address_invoice->company) {
            $data['SEX'] = $customer->id_gender == 1 ? 'M' : 'V';
            $data['BIRTHDATE'] = date('d-m-Y', strtotime($customer->birthday));
        } else {
            $data['COMPANYNAME'] = $address_invoice->company;
        }
        $data['CHAMBEROFCOMMERCE'] = $address_invoice->dni;
        $data['STREET'] = $address_invoice->address1;
        $data['HOUSENUMBER'] = $address_invoice->address2;
        if (preg_match('/[a-zA-Z]$/', trim($address_invoice->address2), $appendix)) {
            $data['HOUSENUMBER'] = str_replace($appendix[0], '', trim($address_invoice->address2));
            $data['HOUSEEXTENSION'] = $appendix[0];
        }
        $data['POSTALCODE'] = $address_invoice->postcode;
        
        $data['COUNTRYCODE'] = $country->iso_code;
        $data['CITY'] = $address_invoice->city;
        $data['DELIVERYSTREET'] = $address_delivery->address1;

        $data['DELIVERYHOUSENUMBER'] = $address_delivery->address2;
        if (preg_match('/[a-zA-Z]$/', trim($address_delivery->address2), $appendix)) {
            $data['DELIVERYHOUSENUMBER'] = str_replace($appendix[0], '', trim($address_delivery->address2));
            $data['DELIVERYHOUSEEXTENSION'] = $appendix[0];
        }

        $data['DELIVERYPOSTALCODE'] = $address_delivery->postcode;
        $country = new Country((int)$address_delivery->id_country);
        $data['DELIVERYCOUNTRYCODE'] = $country->iso_code;

        $data['DELIVERYCITY'] = $address_delivery->city;
        if ($address_delivery->company) {
            $data['DELIVERYADDRESSCOMPANYNAME'] = $address_delivery->company;
        }
        $data['DELIVERYADDRESSFIRSTNAME'] = $address_delivery->firstname;
        $data['DELIVERYADDRESSLASTNAME'] = $address_delivery->lastname;
        $data['PHONENUMBER'] = $address_delivery->phone ? $address_delivery->phone : $address_delivery->phone_mobile;
        $data['EMAIL'] = $customer->email;
        $data['IP'] = $_SERVER['REMOTE_ADDR'];
        if ($address_delivery->other) {
            $data['ADITIONALTEXT'] = $address_delivery->other;
        }
        // test
        $data['ADITIONALTEXT'] = date('Y-m-d H:i:s', time());

        $calculatedTotal = 0;

        $cart = $this->context->cart;
        foreach ($cart->getProducts() as $key => $value) {
            $vat = Tools::ps_round(($value['price_wt']/$value['price']), 2);
            $vat = (int)Tools::ps_round((($vat-1)*100));
            $item = array();
            $item['CODE'] = $value['reference'];
            $item['DESCRIPTION'] = $value['name'] . ' - ' . $value['attributes'];
            $item['ORDERQUANTITY'] = $value['cart_quantity'];
            if ($data['TYPE'] == 'P') {
                $item['PRICEINCL'] = Tools::ps_round($value['price_wt'], 4);
            } else {
                $item['PRICEEXCL'] = Tools::ps_round($value['price'], 4);
            }
            $item['BTW'] = $vat;
            $data['ORDERITEMS'][] = array('ITEM' => $item);
            $calculatedTotal += Tools::ps_round($value['price_wt'], 4)*$value['cart_quantity'];
        }

        if ($cart->getTotalShippingCost()!==0) {
            $vat = $cart->getTotalShippingCost()/$cart->getTotalShippingCost(null, false);
            $vat = (int)Tools::ps_round((($vat-1)*100));
            $item = array();
            $item['CODE'] = $this->module->l('shipping', 'payment');
            $item['DESCRIPTION'] = $this->module->l('Shipping cost', 'payment');
            $item['ORDERQUANTITY'] = 1;
            if ($data['TYPE'] == 'P') {
                $item['PRICEINCL'] = Tools::ps_round($cart->getTotalShippingCost(), 2);
            } else {
                $item['PRICEEXCL'] = Tools::ps_round($cart->getTotalShippingCost(null, false), 2);
            }
            $item['BTW'] = $vat;
            $data['ORDERITEMS'][] = array('ITEM' => $item);
            $calculatedTotal += Tools::ps_round($cart->getTotalShippingCost(), 2);
        }

        if (count($cart->getCartRules())) {
            foreach ($cart->getCartRules() as $rule) {
                $description = ($rule['code']!=='') ? $rule['code'] : $rule['name'];
                $item = array();
                $item['CODE'] = $description;
                $item['DESCRIPTION'] = $rule['description'];
                $item['ORDERQUANTITY'] = 1;
                $item['PRICEEXCL'] = (float)-($rule['value_real']);
                $item['PRICEINCL'] = (float)-($rule['value_real']);
                $item['BTW'] = 0;
                $data['ORDERITEMS'][] = array('ITEM' => $item);
                $calculatedTotal += (float)-($rule['value_real']);
            }
        }

        // adding rounding difference
        if ($calculatedTotal !== $cart_total) {
            $rounding_difference_price = (float)Tools::ps_round($cart_total-$calculatedTotal, 2);
            if ($rounding_difference_price) {
                $data['ORDERITEMS'][] = array(
                    'ITEM' => array(
                        'CODE' => $this->module->l('rounding_difference', 'payment'),
                        'DESCRIPTION' => $this->module->l('Rounding difference', 'payment'),
                        'ORDERQUANTITY' => 1,
                        'PRICEEXCL' => $rounding_difference_price,
                        'PRICEINCL' => $rounding_difference_price,
                        'BTW' => 0,
                    )
                );
            }
        }

        // create billink order and return response
        return $billink->request('/order', $data);
    }
}
